from django.test import TestCase
from django.utils import translation

from .models import MyModel


class TranslatableModelTests(TestCase):
    def test_translation(self):
        animal = MyModel.objects.create()
        animal.translations.create(language_code='en', label='Frog')

        with translation.override('en'):
            self.assertEqual(animal.label, 'Frog')

    def test_override(self):
        animal = MyModel.objects.create()
        animal.translations.create(language_code='en', label='Frog')
        animal.translations.create(language_code='de', label='Frosch')

        with translation.override('en'):
            self.assertEqual(animal.label, 'Frog')
        with translation.override('de'):
            self.assertEqual(animal.label, 'Frosch')

    def test_refresh_from_db(self):
        animal = MyModel.objects.create()
        en = animal.translations.create(language_code='en', label='Frog')

        with translation.override('en'):
            self.assertEqual(animal.label, 'Frog')

        en.label = 'Cow'
        en.save()
        with translation.override('en'):
            self.assertEqual(animal.label, 'Frog')

        animal.refresh_from_db()
        with translation.override('en'):
            self.assertEqual(animal.label, 'Cow')

    def test_unsaved(self):
        animal = MyModel()

        with translation.override('en'):
            self.assertEqual(animal.label, 'not translated')

    def test_no_translation(self):
        animal = MyModel.objects.create()

        with translation.override('en'):
            self.assertEqual(animal.label, 'not translated')

    def test_non_field(self):
        animal = MyModel.objects.create()

        with self.assertRaises(AttributeError):
            animal.does_not_exist



from django.contrib import admin


class TranslatableAdmin(admin.ModelAdmin):
    translation_inline_class = admin.TabularInline

    def get_inlines(self, request, obj):
        class TranslationInline(self.translation_inline_class):
            model = obj.translations.model
            min_num = 2
            extra = 0
        return [
            TranslationInline,
            *super().get_inlines(request, obj),
        ]



from django.db import models
from django.utils.translation import get_language


class TranslatableModel(models.Model):
    class Meta:
        abstract = True

    @property
    def translation(self):
        translation = self.__dict__.get('_translation')
        if translation and translation.language_code == get_language():
            return translation

        if not self.pk:
            raise self.translations.model.DoesNotExist

        translation = self.translations.get(language_code=get_language())
        self.__dict__['_translation'] = translation
        return translation

    def refresh_from_db(self, *args, **kwargs):
        self.__dict__.pop('_translation', None)
        return super().refresh_from_db(*args, **kwargs)

    def __getattr__(self, key):
        fields = self.translations.model._meta.get_fields()
        if key in (f.attname for f in fields):
            try:
                return getattr(self.translation, key)
            except self.translations.model.DoesNotExist:
                return 'not translated'
        else:
            raise AttributeError

    @classmethod
    def get_language_field(cls):
        return models.CharField('Language', max_length=15, db_index=True)

    @classmethod
    def get_parent_field(cls):
        return models.ForeignKey(
            cls,
            on_delete=models.CASCADE,
            editable=False,
            null=True,
            related_name='translations',
        )
